@@ -1,37 +1,18 @@
-/*******************************************************************************
-*
-* University of Illinois/NCSA
-* Open Source License
-*
-* Copyright (c) 2018 Advanced Micro Devices, Inc. All Rights Reserved.
-*
-* Permission is hereby granted, free of charge, to any person obtaining a copy
-* of this software and associated documentation files (the "Software"), to deal
-* with the Software without restriction, including without limitation the
-* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
-* sell copies of the Software, and to permit persons to whom the Software is
-* furnished to do so, subject to the following conditions:
-*
-*     * Redistributions of source code must retain the above copyright notice,
-*       this list of conditions and the following disclaimers.
-*
-*     * Redistributions in binary form must reproduce the above copyright
-*       notice, this list of conditions and the following disclaimers in the
-*       documentation and/or other materials provided with the distribution.
-*
-*     * Neither the names of Advanced Micro Devices, Inc. nor the names of its
-*       contributors may be used to endorse or promote products derived from
-*       this Software without specific prior written permission.
-*
-* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-* CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
-* THE SOFTWARE.
-*
-*******************************************************************************/
+//===- amd_comgr.h.in - User-facing APIs ----------------------------------===//
+//
+// Part of Comgr, under the Apache License v2.0 with LLVM Exceptions. See
+// amd/comgr/LICENSE.TXT in this repository for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+//
+//===----------------------------------------------------------------------===//
+///
+/// \file
+/// This file defines the user-facing Comgr APIs, including compilation,
+/// metadata, and disassembly, symbol lookup, and symbolization APIs.
+///
+/// It is copied into amd_comgr.h by CMake during the Comgr build.
+///
+//===----------------------------------------------------------------------===//
 
 #ifndef AMD_COMGR_H_
 #define AMD_COMGR_H_
@@ -112,7 +93,7 @@ extern "C" {
  * passed to operations must be disjoint, together with all the @p
  * amd_comgr_data_t handles that have been added to it. The exception is that
  * the default device library data object handles can be non-disjoint as they
- * are imutable.
+ * are immutable.
  *
  * The library supports generating and inspecting code objects that
  * contain machine code for a certain set of instruction set
@@ -135,6 +116,15 @@ extern "C" {
  * amd_comgr_do_action to perform an action specified by
  * @p amd_comgr_action_kind_t.
  *
+ * Data objects are reference counted and are destroyed when the
+ * reference count reaches 0. When a data object is created, its
+ * reference count is 1, it has 0 bytes of data, it has an empty name,
+ * and it has no metadata.
+ *
+ * Mutating a data object is only permitted before it is used as part of
+ * the input to an action. A data object which is the result of an action
+ * must not be mutated.
+ *
  * Some data objects can have associated metadata. There are
  * operations for querying this metadata.
  *
@@ -148,6 +138,8 @@ extern "C" {
  *   These files do not appear in the current working directory, but are
  *   instead left in a platform-specific temporary directory (/tmp on Linux and
  *   C:\Temp or the path found in the TEMP environment variable on Windows).
+ * - @p AMD_COMGR_SAVE_LLVM_TEMPS: If this is set, and is not "0", Comgr
+ *   forwards "--save-temps=obj" to Clang Driver invocations
  * - @p AMD_COMGR_REDIRECT_LOGS: If this is not set, or is set to "0", logs are
  *   returned to the caller as normal. If this is set to "stdout"/"-" or
  *   "stderr", logs are instead redirected to the standard output or error
@@ -158,7 +150,6 @@ extern "C" {
  *   include additional Comgr-specific informational messages.
  */
 
-
 /** \defgroup symbol_versions_group Symbol Versions
  *
  * The names used for the shared library versioned symbols.
@@ -568,11 +559,6 @@ amd_comgr_get_isa_metadata(
 /**
  * @brief Create a data object that can hold data of a specified kind.
  *
- * Data objects are reference counted and are destroyed when the
- * reference count reaches 0. When a data object is created its
- * reference count is 1, it has 0 bytes of data, it has an empty name,
- * and it has no metadata.
- *
  * @param[in] kind The kind of data the object is intended to hold.
  *
  * @param[out] data A handle to the data object created. Its reference
@@ -593,12 +579,16 @@ amd_comgr_create_data(
   amd_comgr_data_kind_t kind,
   amd_comgr_data_t *data) AMD_COMGR_VERSION_1_8;
 
- /**
+/**
  * @brief Indicate that no longer using a data object handle.
  *
  * The reference count of the associated data object is
  * decremented. If it reaches 0 it is destroyed.
  *
+ * @note Although this may lead to the destruction of a data object, it is not
+ * considered a mutation for the purposes of the restrictions described in @ref
+ * codeobjectmanager.
+ *
  * @param[in] data The data object to release.
  *
  * @retval ::AMD_COMGR_STATUS_SUCCESS The function has
@@ -644,6 +634,9 @@ amd_comgr_get_data_kind(
  * associated with the data object is also replaced which invalidates
  * all metadata handles to the old metadata.
  *
+ * @warning This function mutates the data object; see @ref codeobjectmanager
+ * for restrictions.
+ *
  * @param[in] data The data object to update.
  *
  * @param[in] size The number of bytes in the data specified by @p bytes.
@@ -673,6 +666,9 @@ amd_comgr_set_data(
  * Internally this API calls amd_comgr_set_data and resets data object's
  * current state.
  *
+ * @warning This function mutates the data object; see @ref codeobjectmanager
+ * for restrictions.
+ *
  * @param[in, out] data The data object to update.
  *
  * @param[in] file_descriptor The native file descriptor for an open file.
@@ -707,6 +703,9 @@ amd_comgr_set_data_from_file_slice(
  * name. The name may also be used for other data objects in log and
  * diagnostic output.
  *
+ * @warning This function mutates the data object; see @ref codeobjectmanager
+ * for restrictions.
+ *
  * @param[in] data The data object to update.
  *
  * @param[in] name A null terminated string that specifies the name to
@@ -1564,9 +1563,6 @@ amd_comgr_action_info_get_working_directory_path(
  *
  * @retval ::AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT @p
  * action_info is an invalid action info object.
- *
- * @retval ::AMD_COMGR_STATUS_ERROR_OUT_OF_RESOURCES
- * Unable to update action info object as out of resources.
  */
 amd_comgr_status_t AMD_COMGR_API
 amd_comgr_action_info_set_logging(
@@ -1585,9 +1581,6 @@ amd_comgr_action_info_set_logging(
  *
  * @retval ::AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT @p
  * action_info is an invalid action info object. @p logging is NULL.
- *
- * @retval ::AMD_COMGR_STATUS_ERROR_OUT_OF_RESOURCES
- * Unable to update the data object as out of resources.
  */
 amd_comgr_status_t AMD_COMGR_API
 amd_comgr_action_info_get_logging(
@@ -1613,17 +1606,18 @@ typedef enum amd_comgr_action_kind_s {
    */
   AMD_COMGR_ACTION_SOURCE_TO_PREPROCESSOR = 0x0,
   /**
-   * Copy all existing data objects in @p input to @p output, then add the
-   * device-specific and language-specific precompiled headers required for
-   * compilation.
+   * Copy all existing data objects in @p input to @p output.
    *
-   * Currently the only supported languages are @p AMD_COMGR_LANGUAGE_OPENCL_1_2
-   * and @p AMD_COMGR_LANGUAGE_OPENCL_2_0.
+   * Currently the action is a no-op, as the OpenCL pre-compiled headers
+   * are no longer used.
    *
-   * Return @p AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT if isa name or language
-   * is not set in @p info, or the language is not supported.
+   * Return @p AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT if any of the
+   * input or output are not initialized.
    */
-  AMD_COMGR_ACTION_ADD_PRECOMPILED_HEADERS = 0x1,
+  AMD_COMGR_ACTION_ADD_PRECOMPILED_HEADERS
+    AMD_COMGR_DEPRECATED("Will be removed in Comgr v4.0. Currently the action\
+    is a no-op, as the OpenCL pre-compiled headers are no longer used.")
+    = 0x1,
   /**
    * Compile each source data object in @p input in order. For each
    * successful compilation add a bc data object to @p result. Resolve
@@ -1719,45 +1713,23 @@ typedef enum amd_comgr_action_kind_s {
    */
   AMD_COMGR_ACTION_ASSEMBLE_SOURCE_TO_RELOCATABLE = 0x8,
   /**
-   * Disassemble each relocatable data object in @p input in
-   * order. For each successful disassembly add a source data object to
-   * @p result.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR if any disassembly
-   * fails.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT
-   * if isa name is not set in @p info and does not match the isa name
-   * of all relocatable data objects in @p input.
+   * @ deprecated
+   * Return @p AMD_COMGR_STATUS_ERROR_ERROR_INVALID_ARGUMENT
    */
-  AMD_COMGR_ACTION_DISASSEMBLE_RELOCATABLE_TO_SOURCE = 0x9,
+  AMD_COMGR_ACTION_DISASSEMBLE_RELOCATABLE_TO_SOURCE AMD_COMGR_DEPRECATED("This\
+      action will return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT") = 0x9,
   /**
-   * Disassemble each executable data object in @p input in order. For
-   * each successful disassembly add a source data object to @p result.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR if any disassembly
-   * fails.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT
-   * if isa name is not set in @p info and does not match the isa name
-   * of all relocatable data objects in @p input.
+   * @ deprecated
+   * Return @p AMD_COMGR_STATUS_ERROR_ERROR_INVALID_ARGUMENT
    */
-  AMD_COMGR_ACTION_DISASSEMBLE_EXECUTABLE_TO_SOURCE = 0xA,
+  AMD_COMGR_ACTION_DISASSEMBLE_EXECUTABLE_TO_SOURCE AMD_COMGR_DEPRECATED("This\
+      action will return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT") = 0xA,
   /**
-   * Disassemble each bytes data object in @p input in order. For each
-   * successful disassembly add a source data object to @p
-   * result. Only simple assembly language commands are generate that
-   * corresponf to raw bytes are supported, not any directives that
-   * control the code object layout, or symbolic branch targets or
-   * names.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR if any disassembly
-   * fails.
-   *
-   * Return @p AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT
-   * if isa name is not set in @p info
+   * @ deprecated
+   * Return @p AMD_COMGR_STATUS_ERROR_ERROR_INVALID_ARGUMENT
    */
-  AMD_COMGR_ACTION_DISASSEMBLE_BYTES_TO_SOURCE = 0xB,
+  AMD_COMGR_ACTION_DISASSEMBLE_BYTES_TO_SOURCE AMD_COMGR_DEPRECATED("This\
+      action will return AMD_COMGR_STATUS_ERROR_INVALID_ARGUMENT") = 0xB,
   /**
    * Compile each source data object in @p input in order. For each
    * successful compilation add a bc data object to @p result. Resolve
@@ -1828,8 +1800,8 @@ typedef enum amd_comgr_action_kind_s {
    * For each successful compilation, add a relocatable object to @p result
    *
    * We accomplish this by first translating the .spv files to .bc via the
-   * SPIR-V translator. We then extract any SPIR-V flags from the embedded
-   * @llvm.cmdline variable. Finally, we compile the bitcode to a relocatable,
+   * SPIR-V translator. We then extract any relevant -cc1 flags from the embedded
+   * @llvm.cmdline variable. Finally, we compile the bitcode to a reloctable,
    * appending any extracted flags.
    *
    * Return @p AMD_COMGR_STATUS_ERROR if any translation, flag extraction, or
@@ -2573,7 +2545,7 @@ amd_comgr_status_t AMD_COMGR_API
 amd_comgr_map_name_expression_to_symbol_name(
     amd_comgr_data_t data,
     size_t *size,
-    char *name_expression,
+    const char *name_expression,
     char *symbol_name) AMD_COMGR_VERSION_2_6;
 
 /**
